
#include <stdio.h>
#include <stdint.h>
#include <stdbool.h>
#include <time.h>
#include <stdlib.h>
#include <gprolog.h>*/
/* #include "/usr/lib/gprolog-1.4.4/include/gprolog.h" */

void date_to_unix(PlFIOArg *stamp, PlFIOArg *year, PlFIOArg *month, PlFIOArg *day)
{
  static const short days_since_years_beginning[12] = /* Anzahl der Tage seit Jahresanfang ohne Tage des aktuellen Monats und ohne Schalttag */
    {0,31,59,90,120,151,181,212,243,273,304,334};

  int leapyears = ((year->value.l-1)-1968)/4 /* leap years since 1970 (not including the current, potential leap year) */
                  - ((year->value.l-1)-1900)/100
                  + ((year->value.l-1)-1600)/400;

  long days_since_1970 = (year->value.l-1970)*365 + leapyears
                      + days_since_years_beginning[month->value.l-1] + day->value.l-1;

  if ( (month->value.l>2) && (year->value.l%4==0 && (year->value.l%100!=0 || year->value.l%400==0)) )
    days_since_1970 += 1; /* +leap day, if year is a leap year */

  stamp->value.l = days_since_1970;
}

void unix_to_date(PlFIOArg *stamp, PlFIOArg *year, PlFIOArg *month, PlFIOArg *day  )
{
  uint32_t d, y, m;


  /* transfer of variable */
  d  = stamp->value.l;
  
    /* Unix time starts in 1970 */
  y      = 1970;


  while(1)
  {
    bool     leapYear   = (y % 4 == 0 && (y % 100 != 0 || y % 400 == 0));
    uint16_t daysInYear = leapYear ? 366 : 365;
    
    if (d >= daysInYear)
    {
      d      -= daysInYear;
      ++y; 
    }
    else
    {
 
      /* calculate the month and day */
      static const uint8_t daysInMonth[12] = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
      for(m = 0; m < 12; ++m)
      {
        uint8_t dim = daysInMonth[m];

        /* add a day to feburary if this is a leap year */
        if (m == 1 && leapYear)
          ++dim;

        if (d >= dim)
          d -= dim;
        else
          break;
      }
      break;
    }
  }

  day->value.l = d + 1;
  month->value.l = m + 1;
  year->value.l = y;
}



PlBool
date_unix(PlFIOArg *stamp, PlFIOArg *year, PlFIOArg *month, PlFIOArg *day)
{
  if (!stamp->is_var)                  /* stamp is not a variable */
    {

      year->unify = PL_TRUE;      /* enforce unif. of year */
      month->unify = PL_TRUE;      /* enforce unif. of month */
      day->unify = PL_TRUE;      /* enforce unif. of day */
 
      unix_to_date(stamp,year,month,day);
      return PL_TRUE;              /* succeed */
    }

  if (year->is_var||month->is_var||day->is_var)               /* Date is also a variable */
    Pl_Err_Instantiation();        /* emit instantiation_error */

  date_to_unix(year,month,day,stamp); /* set stamp */
  return PL_TRUE;                  /* succeed */
}

